package sampleSearchDissociation;

import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;


/**
 * This is a fork from the original DNF class. Used for Vibhav tool ground truth.
 * Note that Vibhav tool requires some modifications, e.g., integer starting from 1, and minimized clauses; that is implemented in "DNFevaluationSampleSearch" 
 * Note that 'probMap' is STATIC, as it is shared among several DNFs, e.g., several tuples in the output of a probabilistic query
 * Thus this "DNF" object is actually a set of unary output tuples and their respective DNFs 
 * @author Wolfgang Gatterbauer
 * @version First Version: March, 2010 
 * @version This Version: May 24, 2013
 */
public class DNF {
	public static HashMap<Integer,Double> probMap;	// static: hashmap tuple id -> prob, since variable ids are shared between DNFs
	public HashSet<Clause> clauses;						// non static: DNF consists of list of 'Clause' objects (each contains a linked list of integer)
	public LinkedList<Integer> tupleIDs;				// non static: a list of all input tuples that are part of the linage for this DNF
	private HashMap<Integer,List<Clause>> invList;	// non static: maps tuple ids to clauses that contain it
	

	/**
	 * Creates a DNF, starting from a set of Clauses, i.e. creates data structures 'clauses', 'tupleIDs', 'invList'.
	 * Does not set the static probabilities probMap.
	 * @param clauseSet
	 * @used by 'PostgreSQL.createLineage(CQ)'
	 */
	public DNF(HashSet<Clause> clauseSet) {
		/* --- 'clauses': add external set of clauses into the internal set of clauses --- */
		this.clauses = new HashSet<Clause>();				
		for (Clause c : clauseSet) {
			if (c.tupleIDs.size() > 0) {
				Clause cc = new Clause(c.tupleIDs);
				cc.minimizeClause();							// minimize this clause before it becomes part of the DNF
				this.clauses.add(cc);
			}
		}
		/* --- 'invList': create a map from tupleID to the clause that contains this tuple --- */
		invList = new HashMap<Integer,List<Clause>>();
		for (Clause c : clauses) {
			for (Integer i : c.tupleIDs) {
				List<Clause> temp;
				if (invList.containsKey(i)) 
					temp = invList.get(i) ;
				else {
					temp = new LinkedList<Clause>();
					invList.put(i, temp);
				}
				temp.add(c);
			}
		}
		/* --- 'tupleIDs': one list of all relevant tupleIDs --- */
		tupleIDs = new LinkedList<Integer>(invList.keySet());
	} 



	/**
	 * Sets the static probability Map (tupleID -> p)
	 * @used by 'SQLserver.createLineage(CQ)'
	 */
	public static void setProb(HashMap<Integer,Double> probMap0) {
		probMap = probMap0 ;
	}



	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		String dnfString = "Phi:  ";
		boolean first = true;
		for (Clause c : clauses) {
			if (first) {
				dnfString += c;
				first = false;
			} else {
				dnfString += ", " + c;
			}
		}
/* below would print also the static probabilities. This is overwhelming when displaying just the lineages for each answer tuple*/		
//		dnfString += "\n" + "Prob: ";
//		first = true;
//		for (int i : probMap.keySet()) {
//			if (first) {
//				dnfString += i + ": " + probMap.get(i);
//				first = false;
//			} else {
//				dnfString += ", " + i + ": " + probMap.get(i);
//			}
//		}
		return dnfString;
	}



	
	/**
	 * Scales all probabilities stored in HashMap probMap by a factor f
	 * @param f
	 * @used by 'Exp_VLDBJ_TPCH_AP_GT_ScaledP.java'
	 */
	public static void scaling(double f) {
		for (int i : probMap.keySet()) {
			double p = f*probMap.get(i);		// get current probability, and scale it
			probMap.put(i,p);						// assign the scaled probability
		}
	} 
	

}
