package sampleSearchDissociation;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Formatter;
import java.util.HashMap;
import java.util.Scanner;



/**
 * Evaluates a DNF with Vibhav Gogate's 'SampleSearch'
 * Detail: Based on discussion with Vibhav 23.7.2013, it became clear that the tool does not need to use approximations for the type of lineages used in the experiments. Hence, the calculated probabilities are exact.
 * Detail: the program allows for "fileNameAddition" to have separate input and output files (in order to speed up evaluations and avoid problems with multiple threats)
 * @author Wolfgang Gatterbauer
 * @version First Version: July 26, 2010
 * @version This Version: Oct 19, 2013
 */
public class DNFevaluationSampleSearch {
	private static String inputFileName = "sampleSearch/temp/input";	// directory and filename for input file
	private static String outputFileName = "sampleSearch/temp/output";	// directory and filename for input file
	private static String programFile = "sampleSearch/samplesearch";	// actual adapted executable for MAC



	/**
	 * Method that takes a DNF object and pretty prints the input file for 'SampleSearch' as specified on July 27, 2010
	 * SampleSearch requires that variable numbers are starting from 1, thus needs to create a new intermediate mapping from original DNF 
	 * @param dnf
	 * @param fileNameAddition (distinguisher)
	 */
	public static void writeInputFile(DNF dnf, String fileNameAddition) {
		String fileName = inputFileName + fileNameAddition + ".txt";
		try {
			/* --- Initialize FileWriter --- */
			FileWriter out = new FileWriter(fileName, false);			// true: append, false: overwrite
			BufferedWriter buf = new BufferedWriter(out);
			Formatter fmt = new Formatter(buf);

			/* --- create transformation map --- */											// SampleSearch requires that variable numbers are starting from 1
			HashMap<Integer,Integer> transformMap = new HashMap<Integer,Integer>();	// Map that maps the n occuring integers to number from 1 ... n (used for writing clauses)
			HashMap<Integer,Integer> inverseMap = new HashMap<Integer,Integer>();	// Other map that maps 1 ... n to the n differeng occuring integers (used for writing probabilities)
			int j = 1;
			for (int i : dnf.tupleIDs) {
				transformMap.put(i, j);
				inverseMap.put(j++, i);
			}

			/* --- first file line contains # of variables and # of clauses --- */
			int n = dnf.tupleIDs.size();				// number of variables: size of LinkedList 'tupleIDs"			
			int m = dnf.clauses.size();				// number of clauses: size of hashset 'clauses'
			//fmt.format("%% Created on (%tF, %tT) %n",Calendar.getInstance(),Calendar.getInstance());	// stores current execution date [but creates errors from 'SampleSearch']
			fmt.format("p cnf " + n + " " + m + "%n");

			/* --- next lines contain, for each clause, transformed variables contained, ended with '0' --- */
			for (Clause c : dnf.clauses) {
				for (int i : c.tupleIDs) {
					fmt.format(transformMap.get(i) + " ");		
				}
				fmt.format("0%n");
			}
			/* --- next lines contain, for each variable, its probability (note that 1-p is used!) --- */
			for (int i = 1 ; i<=n; ++i ) {							// assumes that all n variables 1, ...n are used
				fmt.format("%.9f%n", 1.0 - DNF.probMap.get(inverseMap.get(i)));
			}

			/* ============ close file ============ */
			fmt.format("%n%n");
			fmt.close();

		} catch (Exception e) {
			e.printStackTrace();
		}
	}


	/**
	 */
	public static void writeInputFile(DNF dnf) {
		writeInputFile(dnf, "");
	}





	/**
	 * Method that executes the program on the input file
	 */
	public static long executeProgram(String fileNameAddition) {
		String command = programFile + " " + inputFileName + fileNameAddition + ".txt 100 " + outputFileName  + fileNameAddition + ".txt";
		long start, stop, time = 0;	
		try {																											
			start = System.currentTimeMillis();
			@SuppressWarnings("unused")
			Process pr = Runtime.getRuntime().exec(command);		// executes command
			stop = System.currentTimeMillis();
			time = stop - start;												// !! time is not reliable, because Java thinks SampleSearch has finished, but there is no output file written yet

		} catch (IOException err) {
			err.printStackTrace();
		}
		return time;
	}

	/**
	 */
	public static long executeProgram() {
		return executeProgram("");
	}




	/**
	 * Method that reads the output file and translates, then returns the number into the correct probability
	 * Has some additional loops that idle in case SampleSearch has not yet finished writting to the file
	 */
	public static double readOutput(String fileNameAddition) {
		String fileName = outputFileName + fileNameAddition + ".txt";
		double word = 0;
		int count = 0;
		do	{
			try {
				Scanner input = new Scanner(new File(fileName));
				while (input.hasNext()) {
					word = input.nextDouble();
					count++;
				}
				if (count == 0) {
					try{Thread.currentThread(); Thread.sleep(1);}	// make a break
					catch(InterruptedException ie){}						// required to avoid some I/O errors between executing the program and reading its output
				}

				if (count > 1) {											// should not happen anymore now with separate files for separate tuples, just in case
					throw new IllegalArgumentException("Error: file should contain exactly one word but contains instead " + count + ".");
				}

			} catch (FileNotFoundException err) {					// catch error but ignore it; usually SampleSearch has not yet finished writting the output file
				//				err.printStackTrace();
				try{Thread.currentThread(); Thread.sleep(1);}	// make a break
				catch(InterruptedException ie){}						// required to avoid some I/O errors between executing the program and reading its output
			}

		} while(count==0);

		return 1-Math.pow(10.0, word);		// note that SampleSearch is log model, thus "1-exp()", where exp here means basis 10 
	}



	public static double readOutput() {
		return readOutput("");
	}


}
